﻿using System;
using System.Threading;
using System.IO;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Windows;
using System.Xml;
using System.Xml.Serialization;

using Dicom;
using Dicom.Log;
using Dicom.Network;
using Dicom.Imaging;
using Dicom.Imaging.Codec;
using Dicom.IO.Buffer;


namespace Query_Retrieve_Tool
{
    class CStorSCP
    {


        static string StoragePath = @"C:\Users\Keats\DICOM";

        public void StartDicomServer()
        {
             // preload dictionary to prevent timeouts
            DicomDictionary dict = DicomDictionary.Default;

            // start DICOM server on port 104
            DicomServer<CStoreSCP> server = new DicomServer<CStoreSCP>(104);

        }

        class CStoreSCP : DicomService, IDicomServiceProvider, IDicomCStoreProvider, IDicomCEchoProvider
        {
            private static DicomTransferSyntax[] AcceptedTransferSyntaxes = new DicomTransferSyntax[] {
				DicomTransferSyntax.ExplicitVRLittleEndian,
				DicomTransferSyntax.ExplicitVRBigEndian,
				DicomTransferSyntax.ImplicitVRLittleEndian
			};

            private static DicomTransferSyntax[] AcceptedImageTransferSyntaxes = new DicomTransferSyntax[] {
				// Lossless
				DicomTransferSyntax.JPEGLSLossless,
				DicomTransferSyntax.JPEG2000Lossless,
				DicomTransferSyntax.JPEGProcess14SV1,
				DicomTransferSyntax.JPEGProcess14,
				DicomTransferSyntax.RLELossless,
			
				// Lossy
				DicomTransferSyntax.JPEGLSNearLossless,
				DicomTransferSyntax.JPEG2000Lossy,
				DicomTransferSyntax.JPEGProcess1,
				DicomTransferSyntax.JPEGProcess2_4,

				// Uncompressed
				DicomTransferSyntax.ExplicitVRLittleEndian,
				DicomTransferSyntax.ExplicitVRBigEndian,
				DicomTransferSyntax.ImplicitVRLittleEndian
			};

            public CStoreSCP(Stream stream, Logger log)
                : base(stream, log)
            {
            }

            public void OnReceiveAssociationRequest(DicomAssociation association)
            {
                //Change the AET of the Server here.
                if (association.CalledAE != "KEATSTEST")
                {
                    SendAssociationReject(DicomRejectResult.Permanent, DicomRejectSource.ServiceUser, DicomRejectReason.CalledAENotRecognized);
                    MessageBox.Show("Send Association Rejected ");
                    return;
                }

                foreach (var pc in association.PresentationContexts)
                {
                    if (pc.AbstractSyntax == DicomUID.Verification)
                        pc.AcceptTransferSyntaxes(AcceptedTransferSyntaxes);
                    else if (pc.AbstractSyntax.StorageCategory != DicomStorageCategory.None)
                        pc.AcceptTransferSyntaxes(AcceptedImageTransferSyntaxes);
                }

                SendAssociationAccept(association);
            }

            public void OnReceiveAssociationReleaseRequest()
            {
                SendAssociationReleaseResponse();
            }

            public void OnReceiveAbort(DicomAbortSource source, DicomAbortReason reason)
            {
            }

            public void OnConnectionClosed(int errorCode)
            {
            }

            public DicomCStoreResponse OnCStoreRequest(DicomCStoreRequest request)
            {
                var studyUid = request.Dataset.Get<string>(DicomTag.StudyInstanceUID);
                var instUid = request.SOPInstanceUID.UID;

                var path = Path.GetFullPath(StoragePath);
                path = Path.Combine(path, studyUid);
                //MessageBox.Show("path 1 " + path);


                if (!Directory.Exists(path))
                    Directory.CreateDirectory(path);

                path = Path.Combine(path, instUid) + ".dcm";
                //MessageBox.Show("path 2 " + path);
                request.File.Save(path);

                return new DicomCStoreResponse(request, DicomStatus.Success);
            }

            public void OnCStoreRequestException(string tempFileName, Exception e)
            {
                // let library handle logging and error response
            }

            public DicomCEchoResponse OnCEchoRequest(DicomCEchoRequest request)
            {
                return new DicomCEchoResponse(request, DicomStatus.Success);
            }
        }
    }
}
