﻿using System;
using System.Threading;
using System.IO;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Linq;
using System.Text;
using System.Xml;
using System.Xml.Serialization;
using System.Windows.Forms;
using System.Windows;

using Dicom;
using Dicom.Log;
using Dicom.Network;
using Dicom.Imaging;
using Dicom.Imaging.Codec;
using Dicom.IO.Buffer;

//C.4.2.2.1 Baseline Behavior of SCU
//An SCU conveys the following semantics with a C-MOVE request:


//1690 — The SCU shall interpret responses to the C-MOVE with status equal to Pending
//during the processing of the C-STORE sub-operations. These responses shall
//indicate the number of Remaining, Completed, Failed, and Warning C-STORE suboperations.

//1685 — The SCU shall supply a single value in the Unique Key Attribute for each level above
//the Query/Retrieve level. For the level of retrieve, the SCU shall supply a single value
//for one unique key if the level of retrieve is above the STUDY level and shall supply
//one UID, or a list of UIDs if a retrieval of several items is desired and the retrieve level
//is STUDY, SERIES or IMAGE. The SCU shall also supply a move destination. The
//move destination shall be the DICOM Application Entity Title of a DICOM Application
//Entity capable of serving as the SCP of the Storage Service Class.


//1695 — The SCU shall interpret responses with a status equal to Success, Warning, Failure,
//or Refused as final responses. The final response shall indicate the number of
//Successful C-STORE sub-operations and the number of Failed C-STORE suboperations
//resulting from the C-MOVE operation. The SCU shall interpret a status of: 

//    o Success to indicate that all sub-operations were successfully completed
//    o Warning to indicate one or more sub-operations were successfully completed
//     and one or more sub-operations were unsuccessful or had a status of
//     warning, or all sub-operations had a status of warning
//    o Failure or Refused to indicate all sub-operations were unsuccessful.

namespace Query_Retrieve_Tool
{
    public class CStoreSCU
    {
        public Boolean Success = false;
        public Boolean Pending = false;
        public Boolean Warning = false;
        public Boolean Failure = false;
        public Boolean Cancel = false;

        public string RemainingSuboperations = "";
        public string CompletedSuboperations = "";

        public void CStore_SCU(String remotehost, Int16 port, string callingAET, string calledAET, List<string> ImageList)
        {
           //DicomPriority priority = DicomPriority.High;
            int NumOfImages = ImageList.Count;

            for (int i = 0; i < ImageList.Count; i++) // Loop through List with for
            {
                Success = false;
                Pending = false;
                Warning = false; //RESETS
                Failure = false;
                Cancel = false;

                DicomCStoreRequest cStore_request = new DicomCStoreRequest(ImageList[i]);
                
                

                if (NumOfImages < ImageList.Count)
                {
                    NumOfImages++;
                }

                //RemainingSuboperations = (ImageList.Count - NumOfImages).ToString();
                //CompletedSuboperations = NumOfImages.ToString();

                if (Pending == true)
                {
                    //cStore_request.Command.Add(DicomTag.NumberOfRemainingSuboperations, (ImageList.Count - NumOfImages)); //DicomTag 0000, 1020 'US'
                    //cStore_request.Command.Add(DicomTag.NumberOfCompletedSuboperations, NumOfImages); //DicomTag 0000, 1021 'US'
                    //cStore_request.Command.Add(DicomTag.Status, 0xFF01); //DicomTag 0000, 0900 'US'
                    //cStore_request.Command.Add(DicomTag.NumberOfFailedSuboperations,); //DicomTag 0000, 1022
                    //cStore_request.Command.Add(DicomTag.NumberOfWarningSuboperations,); //DicomTag 0000, 1023

                    MessageBox.Show("Pending DicomTag Status is : " + DicomTag.Status);
                }

                if (Success == true)
                {
                    cStore_request.Dataset.Add(DicomTag.NumberOfRemainingSuboperations, (ImageList.Count - NumOfImages)); //DicomTag 0000, 1020 'US'
                    cStore_request.Dataset.Add(DicomTag.NumberOfCompletedSuboperations, NumOfImages); //DicomTag 0000, 1021 'US'
                    cStore_request.Dataset.Add(DicomTag.Status, 0xFF00); //(ushort)65280); //DicomTag 0000, 0900 'US'

                    MessageBox.Show("Success DicomTag Status is : " + Success);
                }


                DicomCStoreRequest.ResponseDelegate cStoreResponseDelegate = CStoreResponse;
                cStore_request.OnResponseReceived = cStoreResponseDelegate;

                DicomClient client = new DicomClient();
                client.AddRequest(cStore_request);
                client.Send(remotehost, port, false, callingAET, calledAET);
                //ThreadPool.QueueUserWorkItem(new WaitCallback(SendImages));
            } 
        }


        public void CStoreResponse(DicomCStoreRequest request, DicomCStoreResponse response)
        {
            //Responses are from SCP Machine

            if (response.Status.ToString() == "Pending")
            {
                Pending = true;
                MessageBox.Show("Pending DicomTag Status is : " + DicomTag.Status.ToString() + "  " + response.Status.ToString());
                //MessageBox.Show("Pending DicomTag Status is : " + response.Status.ToString());
            }

            if (response.Status.ToString() == "Success")
            {
                Success = true;
                MessageBox.Show("Success DicomTag Status is : " + DicomTag.Status.ToString() + "  " + response.Status.ToString());
                //MessageBox.Show("Success DicomTag Status is : " + response.Status.ToString());
            }

            //response.Dataset.Get<string>(DicomTag.OffendingElement); //DicomTag 0000, 0901
            //response.Dataset.Get<string>(DicomTag.ErrorComment); //DicomTag 0000, 0902

            //MessageBox.Show("response is : " + response.Status);
        }

    }
}
